<!DOCTYPE xsl:stylesheet [

	<!ENTITY degrees "&#176;">

]>
<!--

	Stylesheet to convert TEI to HTML

	Developed by Jeroen Hellingman <jeroen@bohol.ph>, to be used together with a CSS stylesheet. Please contact me if you have problems with this stylesheet, or have improvements or bug fixes to contribute.

	This stylesheet can be used with the saxon XSL processor.

	Embed this style sheet in the source document using the <?xml-stylesheet type="text/xsl" href="stylesheet.xsl"?> processing instruction. This works with IE 6.0 or the latest Mozilla browsers.

	This file is hereby dedicated to the public domain.

-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
	<xsl:output
		doctype-public="-//W3C//DTD HTML 4.01 Transitional//EN"
		doctype-system="http://www.w3.org/TR/html4/loose.dtd"
		method="html"
		encoding="UTF-8"/>

	<!--====================================================================-->

	<xsl:variable name="optionExternalCSS" select="'No'" />
	<xsl:variable name="optionPGHeaders"   select="'No'" />

	<!--====================================================================-->

	<xsl:variable name="title" select="/TEI.2/teiHeader/fileDesc/titleStmt/title" />
	<xsl:variable name="author" select="/TEI.2/teiHeader/fileDesc/titleStmt/author" />
	<xsl:variable name="pubdate" select="/TEI.2/teiHeader/fileDesc/publicationStmt/date" />

	<xsl:variable name="language" select="/TEI.2/@lang" />
	<xsl:variable name="baselanguage" select="substring-before($language,'-')" />
	<xsl:variable name="defaultlanguage" select="'en'" />

	<xsl:variable name="messages" select="document('messages.xml')/repository"/>

	<xsl:variable name="strUnitsUsed" select="'Original'"/>

	<!--====================================================================-->

	<xsl:template name="GetMessage">
		<xsl:param name="name" select="'msgError'"/>
		<xsl:variable name="msg" select="$messages/messages/message[@name=$name]"/>
		<xsl:choose>
			<xsl:when test="$msg[lang($language)][1]">
				<xsl:apply-templates select="$msg[lang($language)][1]"/>
			</xsl:when>
			<xsl:when test="$msg[lang($baselanguage)][1]">
				<xsl:apply-templates select="$msg[lang($baselanguage)][1]"/>
			</xsl:when>
			<xsl:when test="$msg[lang($defaultlanguage)][1]">
				<xsl:message terminate="no">
					Warning: message '<xsl:value-of select="$name"/>' not available in locale <xsl:value-of select="$language"/>.
				</xsl:message>
				<xsl:apply-templates select="$msg[lang($defaultlanguage)][1]"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message terminate="no">
					Warning: unknown message '<xsl:value-of select="$name"/>'.
				</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template name="FormatMessage">
		<xsl:param name="name" select="'msgError'"/>
		<xsl:param name="params"/>
		<xsl:variable name="msg" select="$messages/messages/message[@name=$name]"/>
		<xsl:choose>
			<xsl:when test="$msg[lang($language)][1]">
				<xsl:apply-templates select="$msg[lang($language)][1]" mode="formatMessage">
					<xsl:with-param name="params" select="$params"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="$msg[lang($baselanguage)][1]">
				<xsl:apply-templates select="$msg[lang($baselanguage)][1]" mode="formatMessage">
					<xsl:with-param name="params" select="$params"/>
				</xsl:apply-templates>
			</xsl:when>
			<xsl:when test="$msg[lang($defaultlanguage)][1]">
				<xsl:message terminate="no">
					Warning: message '<xsl:value-of select="$name"/>' not available in locale <xsl:value-of select="$language"/>.
				</xsl:message>
				<xsl:apply-templates select="$msg[lang($defaultlanguage)][1]" mode="formatMessage"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:message terminate="no">
					Warning: unknown message '<xsl:value-of select="$name"/>'.
				</xsl:message>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>

	<xsl:template match="*" mode="formatMessage">
		<xsl:param name="params"/>
			<xsl:copy>
				<xsl:copy-of select="@*"/>
				<xsl:apply-templates mode="formatMessage">
					<xsl:with-param name="params" select="$params"/>
				</xsl:apply-templates>
			</xsl:copy>
	</xsl:template>

	<!--
	<xsl:template xmlns:msxsl="urn:schemas-microsoft-com:xslt" match="param" mode="formatMessage">
		<xsl:param name="params"/>
		<xsl:choose>
			<xsl:when test="msxsl:node-set($params)/params/param[@name=current()/@name]">
				<xsl:value-of select="msxsl:node-set($params)/params/param[@name=current()/@name]"/>
			</xsl:when>
			<xsl:otherwise>
				[### <xsl:value-of select="@name"/> ###]
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	-->

	<xsl:template match="param" mode="formatMessage">
		<xsl:param name="params"/>
		<xsl:choose>
			<xsl:when test="$params/params/param[@name=current()/@name]">
				<xsl:value-of select="$params/params/param[@name=current()/@name]"/>
			</xsl:when>
			<xsl:otherwise>
				[### <xsl:value-of select="@name"/> ###]
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>


	<xsl:template match="space" mode="formatMessage">
		<xsl:text> </xsl:text>
	</xsl:template>

	<!--====================================================================-->
	<!-- Pull strings into easier to use variables -->

	<xsl:variable name="strFigure">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgFigure'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strPart">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgPart'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strTableOfContents">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgTableOfContents'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strAppendix">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgAppendix'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strPage">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgPage'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strChapter">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgChapter'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strBook">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgBook'"/>
		</xsl:call-template>
	</xsl:variable>	<xsl:variable name="strAnd">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgAnd'"/>
		</xsl:call-template>
	</xsl:variable>
	<xsl:variable name="strListOfIllustrations">
		<xsl:call-template name="GetMessage">
			<xsl:with-param name="name" select="'msgListOfIllustrations'"/>
		</xsl:call-template>
	</xsl:variable>

	<!--====================================================================-->

	<xsl:template match="/">
		<xsl:comment>
			<xsl:text> This HTML file has been automatically generated from an XML source, using XSLT. If you find any mistakes, please edit the XML source. </xsl:text>
		</xsl:comment>
		<xsl:text> <!-- insert extra new-line for PG -->
		</xsl:text>
		<html lang="{$language}">
			<head>
				<title>
					<xsl:value-of select="$title"/>
				</title>

				<link rel="schema.DC"     href="http://dublincore.org/documents/1998/09/dces/"/> <!-- WAS: http://purl.org/DC/elements/1.0/ -->

				<meta name="author"       content="{$author}"/>
				<meta name="DC.Creator"   content="{$author}"/>
				<meta name="DC.Title"     content="{$title}"/>
				<meta name="DC.Date"      content="{$pubdate}"/>
				<meta name="DC.Language"  content="{$language}"/>				

				<xsl:if test="$optionExternalCSS = 'Yes'">
					<link href="style/arctic.css"		rel="stylesheet"			type="text/css" title="Artic Blue"/>
					<link href="style/amazonia.css"		rel="alternate stylesheet"	type="text/css" title="Amazonia Green"/>
					<link href="style/borneo.css"		rel="alternate stylesheet"	type="text/css" title="Borneo Brown"/>
					<link href="style/gutenberg.css"	rel="stylesheet"			type="text/css"/>

					<link href="style/print.css"		rel="stylesheet"			type="text/css" media="print"/>
				</xsl:if>

				<xsl:if test="$optionExternalCSS = 'No'">
					<!-- Pull in CSS sheets. This requires them to be wrapped in a tag at toplevel, so they become valid XML -->
					<style type="text/css">
						<xsl:variable name="stylesheet" select="document('style/gutenberg.css.xml')"/>
						<xsl:copy-of select="$stylesheet/*/node()"/>
						<xsl:variable name="stylesheet2" select="document('style/arctic.css.xml')"/>
						<xsl:copy-of select="$stylesheet2/*/node()"/>
					</style>
				</xsl:if>

			</head>
			<body><xsl:text> <!-- insert extra new-line for PG -->
			</xsl:text>
				<xsl:if test="$optionPGHeaders = 'Yes'">
					<xsl:call-template name="PGHeader"/>
				</xsl:if>
				<xsl:apply-templates/>
				<xsl:if test="$optionPGHeaders = 'Yes'">
					<xsl:call-template name="PGFooter"/>
				</xsl:if>
			</body>
		</html><xsl:text> <!-- insert extra new-line for PG -->
		</xsl:text>
	</xsl:template>

	<!--====================================================================-->
	<!-- TEI Header -->

	<!-- Suppress the header in the output -->

	<xsl:template match="teiHeader"/>

	<!--====================================================================-->
	<!-- Title Page -->

	<xsl:template match="titlePage">
		<xsl:apply-templates mode="titlePage"/>
	</xsl:template>

	<xsl:template match="docTitle" mode="titlePage">
		<xsl:apply-templates mode="titlePage"/>
	</xsl:template>

	<xsl:template match="titlePart" mode="titlePage">
		<h1 class="docTitle">
			<xsl:apply-templates mode="titlePage"/>
		</h1>
	</xsl:template>

	<xsl:template match="titlePart[@type='sub']" mode="titlePage">
		<h2 class="docTitle">
			<xsl:apply-templates mode="titlePage"/>
		</h2>
	</xsl:template>

	<xsl:template match="byline" mode="titlePage">
		<h2 class="byline">
			<xsl:apply-templates mode="titlePage"/>
		</h2>
	</xsl:template>

	<xsl:template match="docAuthor" mode="titlePage">
		<span class="docAuthor">
			<xsl:apply-templates mode="titlePage"/>
		</span>
	</xsl:template>

	<xsl:template match="lb" mode="titlePage">
		<br/>
	</xsl:template>

	<xsl:template match="docImprint" mode="titlePage">
		<h2 class="docImprint">
			<xsl:apply-templates/>
		</h2>
	</xsl:template>

	<!--====================================================================-->
	<!-- Corrections -->

	<xsl:template match="corr">
		<xsl:apply-templates/>
	</xsl:template>

	<!--====================================================================-->
	<!-- Cross References -->

	<!-- Only use generated ID's to avoid clashes of original with generated ID's. Note that the target id generated here should also be generated on the element being referenced. We cannot use the id() function here, since we do not use a DTD. -->

	<!-- Special case: reference to footnote, used when the same footnote reference mark is used multiple times -->

	<xsl:template match="ref[@target and @type='noteref']">
		<xsl:variable name="target" select="./@target"/>
		<xsl:apply-templates select="//*[@id=$target]" mode="noterefnumber"/>
	</xsl:template>

	<xsl:template match="note" mode="noterefnumber">
		<a href="#{generate-id()}" class="noteref">
			<xsl:number level="any" count="note[@place='foot' or not(@place)]" from="div1[not(ancestor::q)]"/>
		</a>
	</xsl:template>

	<!-- Normal case -->

	<xsl:template match="ref[@target and not(@type='noteref')]">
		<xsl:variable name="target" select="./@target"/>
		<a id="{generate-id(.)}" href="#{generate-id(//*[@id=$target])}">
			<xsl:apply-templates/>
		</a>
	</xsl:template>

	<!--====================================================================-->
	<!-- Page Breaks -->

	<xsl:template match="pb">
		<span class="pageno">
			<a id="{generate-id()}"/>
			<xsl:if test="@n">
				<xsl:value-of select="$strPage"/><xsl:text> </xsl:text><xsl:value-of select="@n"/>
			</xsl:if>
		</span>
	</xsl:template>

	<!--====================================================================-->
	<!-- Table of Contents -->
	<!-- Take care only to generate ToC entries for divisions of the main text, not for those in quoted texts -->

	<xsl:template match="divGen[@type='toc']">
		<p class="div1"/>
		<h1><xsl:value-of select="$strTableOfContents"/></h1>
		<ul>
			<xsl:apply-templates mode="gentoc" select="/TEI.2/text/front/div1"/>
			<xsl:choose>
				<xsl:when test="/TEI.2/text/body/div0">
					<xsl:apply-templates mode="gentoc" select="/TEI.2/text/body/div0"/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:apply-templates mode="gentoc" select="/TEI.2/text/body/div1"/>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:apply-templates mode="gentoc" select="/TEI.2/text/back/div1"/>
		</ul>
	</xsl:template>

	<xsl:template match="div0" mode="gentoc">
		<xsl:if test="head">
			<li>
				<a href="#{generate-id()}">
				<xsl:choose>
					<xsl:when test="@type='part'">
						<xsl:value-of select="$strPart"/><xsl:text> </xsl:text><xsl:value-of select="./@n"/>:<xsl:text> </xsl:text>
					</xsl:when>
				</xsl:choose>
				<xsl:apply-templates select="head[not(@type='label')]" mode="tochead"/></a>
				<xsl:if test="div1">
					<ul>
						<xsl:apply-templates select="div1" mode="gentoc"/>
					</ul>
				</xsl:if>
			</li>
		</xsl:if>
	</xsl:template>

	<xsl:template match="div1" mode="gentoc">
		<xsl:if test="head">
			<li>
				<a href="#{generate-id()}">
				<xsl:choose>
					<xsl:when test="@type='chapter'">
						<xsl:value-of select="$strChapter"/><xsl:text> </xsl:text><xsl:value-of select="./@n"/>:<xsl:text> </xsl:text>
					</xsl:when>
					<xsl:when test="@type='appendix'">
						Appendix <xsl:value-of select="./@n"/>:<xsl:text> </xsl:text>
					</xsl:when>
				</xsl:choose>
				<xsl:apply-templates select="head[not(@type='label')]" mode="tochead"/></a>
				<xsl:if test="div2">
					<ul>
						<xsl:apply-templates select="div2" mode="gentoc"/>
					</ul>
				</xsl:if>
			</li>
		</xsl:if>
	</xsl:template>

	<xsl:template match="div2" mode="gentoc">
		<xsl:if test="head">
			<li>
				<a href="#{generate-id()}">
				<xsl:apply-templates select="head[not(@type='label')]" mode="tochead"/></a>
				<xsl:if test="div3">
					<ul>
						<xsl:apply-templates select="div3" mode="gentoc"/>
					</ul>
				</xsl:if>
			</li>
		</xsl:if>
	</xsl:template>

	<xsl:template match="div3" mode="gentoc">
		<xsl:if test="head">
			<li>
				<a href="#{generate-id()}">
				<xsl:apply-templates select="head[not(@type='label')]" mode="tochead"/></a>
			</li>
		</xsl:if>
	</xsl:template>


	<!-- Suppress notes in table of contents (to avoid getting them twice) -->

	<xsl:template match="note" mode="tochead"/>


	<!-- Suppress 'label' headings in table of contents -->

	<xsl:template match="head[@type='label']" mode="tochead"/>


	<xsl:template match="head" mode="tochead">
		<xsl:if test="position() = 1 and ./@n">
			<xsl:value-of select="./@n"/><xsl:text> </xsl:text>
		</xsl:if>
		<xsl:if test="position() &gt; 1">
			<xsl:text>: </xsl:text>
		</xsl:if>
		<xsl:apply-templates mode="tochead"/>
	</xsl:template>


	<!-- Text styles in chapter headings -->

	<xsl:template match="hi" mode="tochead">
		<i>
			<xsl:apply-templates mode="tochead"/>
		</i>
	</xsl:template>

	<xsl:template match="hi[@rend='italic']" mode="tochead">
		<i>
			<xsl:apply-templates  mode="tochead"/>
		</i>
	</xsl:template>


	<!-- Generate a gallery of images in an eBook -->

	<xsl:template match="divGen[@type='gallery' or @type='Gallery']">
		<h2><xsl:value-of select="$strListOfIllustrations"/></h2>
		<table>
			<xsl:call-template name="splitrows">
				<xsl:with-param name="figures" select="//figure" />
				<xsl:with-param name="columns" select="3" />
			</xsl:call-template>
		</table>
	</xsl:template>


	<xsl:template name="splitrows">
		<xsl:param name="figures"/>
		<xsl:param name="columns"/>
		<xsl:for-each select="$figures[position() mod $columns = 1]">
			<tr>
				<xsl:apply-templates select=". | following::figure[position() &lt; $columns]" mode="gallery"/>
			</tr>
			<tr>
				<xsl:apply-templates select=". | following::figure[position() &lt; $columns]" mode="gallery-captions"/>
			</tr>
		</xsl:for-each>
	</xsl:template>


	<xsl:template match="figure" mode="gallery">
		<td align="center" valign="center">
			<a href="#{generate-id()}">
				<img border="0">
					<xsl:attribute name="src">images/thumbs/<xsl:value-of select="@id"/>.jpg</xsl:attribute>
					<xsl:attribute name="alt"><xsl:value-of select="head"/></xsl:attribute>
				</img>
			</a>
		</td>
	</xsl:template>


	<xsl:template match="figure" mode="gallery-captions">
		<td align="center" valign="top">
			<a href="#{generate-id()}"><xsl:apply-templates select="head" mode="gallery-captions"/></a>
		</td>
	</xsl:template>


	<xsl:template match="head" mode="gallery-captions">
		<xsl:value-of select="."/>
	</xsl:template>


	<!--====================================================================-->
	<!-- Divisions and Headings -->

	<xsl:template name="GenerateLabel">
		<xsl:param name="headingLevel" select="'h1'"/>
		<xsl:if test="contains(./@rend, 'label(yes)')">
			<xsl:element name="{$headingLevel}">
				<xsl:attribute name="class">label</xsl:attribute>
				<xsl:call-template name="TranslateType"/>
				<xsl:value-of select="@n"/>
			</xsl:element>
		</xsl:if>
	</xsl:template>

	<xsl:template name="TranslateType">
		<xsl:choose>
			<xsl:when test="@type='Appendix' or @type='appendix'">
				<xsl:value-of select="$strAppendix"/><xsl:text> </xsl:text>
			</xsl:when>
			<xsl:when test="@type='Chapter' or @type='chapter'">
				<xsl:value-of select="$strChapter"/><xsl:text> </xsl:text>
			</xsl:when>
			<xsl:when test="@type='Part' or @type='part'">
				<xsl:value-of select="$strPart"/><xsl:text> </xsl:text>
			</xsl:when>
			<xsl:when test="@type='Book' or @type='book'">
				<xsl:value-of select="$strBook"/><xsl:text> </xsl:text>
			</xsl:when>
		</xsl:choose>
	</xsl:template>

	<!-- div0 -->

	<xsl:template match="div0">
		<p class="div0"><a id="{generate-id(.)}"></a></p>
		<xsl:call-template name="GenerateLabel"/>
		<xsl:apply-templates/>
		<xsl:if test=".//note[@place='foot' or not(@place)] and not(ancestor::q) and not(.//div1)">
			<p/><hr class="noteseparator"/>
			<xsl:apply-templates mode="footnotes" select=".//note[@place='foot' or not(@place)]"/>
		</xsl:if>
	</xsl:template>

	<xsl:template match="div0/head">
		<h1>
			<xsl:if test="@type='label'">
				<xsl:attribute name="class">label</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</h1>
	</xsl:template>

	<!-- div1 -->

	<xsl:template match="div1">
		<p class="div1"><a id="{generate-id(.)}"></a></p>
		<xsl:call-template name="GenerateLabel"/>
		<xsl:apply-templates/>
		<xsl:if test=".//note[@place='foot' or not(@place)] and not(ancestor::q)">
			<p/><hr class="noteseparator"/>
			<xsl:apply-templates mode="footnotes" select=".//note[@place='foot' or not(@place)]"/>
		</xsl:if>
	</xsl:template>

	<xsl:template match="div1/head">
		<h1>
			<xsl:if test="@type='label'">
				<xsl:attribute name="class">label</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</h1>
	</xsl:template>

	<!-- div2 -->

	<xsl:template match="div2">
		<p class="div2"><a id="{generate-id(.)}"></a></p>
		<xsl:call-template name="GenerateLabel">
			<xsl:with-param name="headingLevel" select="'h2'"/>
		</xsl:call-template>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="div2/head">
		<h2>
			<xsl:if test="@type='label'">
				<xsl:attribute name="class">label</xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</h2>
	</xsl:template>

	<!-- div3 -->

	<xsl:template match="div3">
		<p class="div3"><a id="{generate-id(.)}"></a></p>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="div3/head">
		<h3><xsl:apply-templates/></h3>
	</xsl:template>

	<!-- div4 -->

	<xsl:template match="div4">
		<p class="div4"><a id="{generate-id(.)}"></a></p>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="div4/head">
		<h4><xsl:apply-templates/></h4>
	</xsl:template>

	<!-- div5 -->

	<xsl:template match="div5">
		<p class="div5"><a id="{generate-id(.)}"></a></p>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="div5/head">
		<h5><xsl:apply-templates/></h5>
	</xsl:template>

	<!-- other headers -->

	<xsl:template match="head">
		<h5><xsl:apply-templates/></h5>
	</xsl:template>

	<xsl:template match="byline">
		<p align="left" class="byline">
			<i><xsl:apply-templates/></i>
		</p>
	</xsl:template>

	<!--====================================================================-->
	<!-- Tables: TODO: improve this for more complicated tables. -->

	<xsl:template match="table">
		<!-- insert </p> to close current paragraph as tables in paragraphs are illegal in HTML -->
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;/p&gt;</xsl:text>
		</xsl:if>
		<xsl:apply-templates select="head"/>
		<table id="{generate-id(.)}" width="100%">
			<xsl:apply-templates select="row"/>
		</table>
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;p&gt;</xsl:text>
		</xsl:if>
	</xsl:template>


	<xsl:template match="table/head">
		<h5 class="tablehead"><xsl:apply-templates/></h5>
	</xsl:template>


	<xsl:template match="row">
		<tr valign="top">
			<xsl:apply-templates/>
		</tr>
	</xsl:template>


	<xsl:template match="row[@role='label']/cell">
		<td valign="top">
			<xsl:call-template name="cell-span"/>
			<b><xsl:apply-templates/></b>
		</td>
	</xsl:template>

	<xsl:template match="cell[@role='label']">
		<td valign="top">
			<xsl:call-template name="cell-span"/>
			<b><xsl:apply-templates/></b>
		</td>
	</xsl:template>


	<xsl:template match="row[@role='unit']/cell">
		<td valign="top">
			<xsl:call-template name="cell-span"/>
			<i><xsl:apply-templates/></i>
		</td>
	</xsl:template>

	<xsl:template match="cell[@role='unit']">
		<td valign="top">
			<xsl:call-template name="cell-span"/>
			<i><xsl:apply-templates/></i>
		</td>
	</xsl:template>


	<xsl:template match="cell">
		<td valign="top">
			<xsl:call-template name="cell-span"/>
			<xsl:apply-templates/>
		</td>
	</xsl:template>


	<xsl:template name="cell-span">
		<xsl:if test="@cols and (@cols > 1)">
			<xsl:attribute name="colspan"><xsl:value-of select="@cols"/></xsl:attribute>
		</xsl:if>
		<xsl:if test="@rows and (@rows > 1)">
			<xsl:attribute name="rowspan"><xsl:value-of select="@rows"/></xsl:attribute>
		</xsl:if>
	</xsl:template>


	<!--====================================================================-->
	<!-- Lists -->

	<xsl:template match="list">
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;/p&gt;</xsl:text>
		</xsl:if>
		<xsl:choose>
			<xsl:when test="contains(@rend, 'columns(2)')">
				<xsl:call-template name="doubleuplist"/>
			</xsl:when>
			<xsl:otherwise>
				<ul id="{generate-id(.)}">
					<xsl:apply-templates/>
				</ul>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;p&gt;</xsl:text>
		</xsl:if>
	</xsl:template>


	<xsl:template name="doubleuplist">
		<table id="{generate-id(.)}">
			<xsl:variable name="halfway" select="ceiling(count(item) div 2)"/>
			<tr valign="top">
				<td>
					<ul>
						<xsl:apply-templates select="item[position() &lt; $halfway + 1]"/>
					</ul>
				</td>
				<td>
					<ul>
						<xsl:apply-templates select="item[position() &gt; $halfway]"/>
					</ul>
				</td>
			</tr>
		</table>
	</xsl:template>


	<xsl:template match="item">
		<li id="{generate-id(.)}">
			<xsl:apply-templates/>
		</li>
	</xsl:template>


	<!--====================================================================-->
	<!-- Figures -->

	<!-- Derive the file name from the unique id, and assume that the format is .jpg. This is a temporary solution. -->

	<xsl:template name="insertimage">
		<xsl:param name="alt" select="''"/>
		<xsl:param name="format" select="'.jpg'"/>

		<xsl:choose>
			<xsl:when test="contains(@rend, 'link(')">
				<a>
					<xsl:attribute name="href">
						<xsl:value-of select="substring-before(substring-after(@rend, 'link('), ')')"/>
					</xsl:attribute>
					<xsl:call-template name="insertimage2">
						<xsl:with-param name="alt" select="$alt"/>
						<xsl:with-param name="format" select="$format"/>
					</xsl:call-template>
				</a>
			</xsl:when>
			<xsl:otherwise>
				<xsl:call-template name="insertimage2">
					<xsl:with-param name="alt" select="$alt"/>
					<xsl:with-param name="format" select="$format"/>
				</xsl:call-template>
			</xsl:otherwise>		
		</xsl:choose>
	</xsl:template>


	<xsl:template name="insertimage2">
		<xsl:param name="alt" select="''"/>
		<xsl:param name="format" select="'.jpg'"/>
		<img border="0">
			<xsl:attribute name="src">
				<xsl:choose>
					<xsl:when test="contains(@rend, 'image(')">
						<xsl:value-of select="substring-before(substring-after(@rend, 'image('), ')')"/>
					</xsl:when>
					<xsl:when test="@url">
						<xsl:value-of select="@url"/>
					</xsl:when>
					<xsl:otherwise>
						images/<xsl:value-of select="@id"/><xsl:value-of select="$format"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>

			<xsl:attribute name="alt">
				<xsl:value-of select="$alt"/>
			</xsl:attribute>
		</img>
	</xsl:template>


	<xsl:template match="figure[@rend='inline']">
		<xsl:call-template name="insertimage">
			<xsl:with-param name="format" select="'.png'"/>
		</xsl:call-template>
	</xsl:template>


	<xsl:template match="figure[@rend='initial']">
		<table align="left" style="margin:0px;"><tr><td>
			<xsl:call-template name="insertimage">
				<xsl:with-param name="format" select="'.gif'"/>
				<xsl:with-param name="alt" select="head"/>
			</xsl:call-template>
		</td></tr></table>
	</xsl:template>


	<xsl:template match="figure">
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;/p&gt;</xsl:text>
		</xsl:if>
		<div id="{generate-id()}">
			<xsl:choose>
				<xsl:when test="@rend='left' or contains(@rend, 'float(left)')">
					<xsl:attribute name="class">divFigure floatLeft</xsl:attribute>
				</xsl:when>
				<xsl:when test="@rend='right' or contains(@rend, 'float(right)')">
					<xsl:attribute name="class">divFigure floatRight</xsl:attribute>
				</xsl:when>
				<xsl:otherwise>
					<xsl:attribute name="class">divFigure</xsl:attribute>
				</xsl:otherwise>
			</xsl:choose>
			<xsl:if test="@n">
				<h3><xsl:value-of select="$strFigure"/><xsl:text> </xsl:text><xsl:value-of select="@n"/>.</h3>
			</xsl:if>
			<p class="legend">
				<xsl:call-template name="insertimage">
					<xsl:with-param name="alt" select="head"/>
				</xsl:call-template>
			</p>
			<xsl:apply-templates/>
		</div>
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;p&gt;</xsl:text>
		</xsl:if>
	</xsl:template>


	<xsl:template match="figure/head">
		<p class="figureHead"><xsl:apply-templates/></p>
	</xsl:template>

	<!--====================================================================-->
	<!-- Arguments (short summary of contents at start of chapter) -->

	<xsl:template match="argument">
		<div class="blockquote">
			<xsl:apply-templates/>
		</div>
	</xsl:template>

	<xsl:template match="epigraph">
		<div class="blockquote">
			<xsl:apply-templates/>
		</div>
	</xsl:template>

	<!--====================================================================-->
	<!-- Blockquotes -->

	<xsl:template match="q[@rend='block']">
		<!-- insert </p> to close current paragraph as nested paragraphs are illegal in HTML -->
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;/p&gt;</xsl:text>
		</xsl:if>
		<div class="blockquote">
			<xsl:apply-templates/>
		</div>
		<xsl:if test="parent::p">
			<xsl:text disable-output-escaping="yes">&lt;p&gt;</xsl:text>
		</xsl:if>
	</xsl:template>

	<!--====================================================================-->
	<!-- Notes -->

	<!-- Marginal notes should go to the margin -->

	<xsl:template match="/TEI.2/text//note[@place='margin']">
		<span class="leftnote">
			<xsl:apply-templates/>
		</span>
	</xsl:template>

	<!-- BUG: notes get counted, even if they have @place='margin' -->

	<xsl:template match="side">
		<span class="leftnote">
			<xsl:apply-templates/>
		</span>
	</xsl:template>

	<!-- Move footnotes to the end of the div1 element they appear in (but not in quoted texts) -->

	<xsl:template match="/TEI.2/text//note[@place='foot' or not(@place)]">
		<a id="{generate-id()}src" href="#{generate-id()}" class="noteref">
			<xsl:number level="any" count="note[@place='foot' or not(@place)]" from="div1[not(ancestor::q)]"/>
		</a>
	</xsl:template>

	<!-- Handle notes with paragraphs different from simple notes -->

	<xsl:template match="note[p]" mode="footnotes">
		<div class="notetext">
			<p class="notetext">
				<a id="{generate-id()}" href="#{generate-id()}src" class="noteref">
					<xsl:number level="any" count="note[@place='foot' or not(@place)]" from="div1[not(ancestor::q)]"/>
				</a>
				<xsl:text> </xsl:text>
				<xsl:apply-templates select="*[1]" mode="footfirst"/>
			</p>
			<xsl:apply-templates select="*[position() > 1]" mode="footnotes"/>
		</div>
	</xsl:template>

	<xsl:template match="note" mode="footnotes">
		<div class="notetext">
			<p class="notetext">
				<a id="{generate-id()}" href="#{generate-id()}src" class="noteref">
					<xsl:number level="any" count="note[@place='foot' or not(@place)]" from="div1[not(ancestor::q)]"/>
				</a>
				<xsl:text> </xsl:text>
				<xsl:apply-templates/>
			</p>
		</div>
	</xsl:template>

	<xsl:template match="*" mode="footfirst">
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="p" mode="footnotes">
		<p id="{generate-id()}" class="notetext"><xsl:apply-templates/></p>
	</xsl:template>

	<xsl:template match="*" mode="footnotes">
		<xsl:apply-templates/>
	</xsl:template>

	<!--====================================================================-->
	<!-- Paragraphs -->

	<xsl:template match="p">
		<p id="{generate-id()}">
			<xsl:if test="contains(@rend, 'align(')">
				<xsl:variable name="align" select="substring-before(substring-after(@rend, 'align('), ')')"/>
				<xsl:attribute name="class">align<xsl:value-of select="$align"/></xsl:attribute>
			</xsl:if>
			<xsl:apply-templates/>
		</p>
	</xsl:template>

    <!-- Special types of Paragraphs (introduced for Filippino Riddles) -->

	<xsl:template match="p[@rend='Question']">
		<xsl:if test="@n">
			<h3 id="{generate-id()}"><xsl:value-of select="@n"/></h3>
		</xsl:if>
		<p class="question"><xsl:apply-templates/></p>
	</xsl:template>

	<xsl:template match="p[@rend='Answer']">
		<p id="{generate-id()}" class="answer"><xsl:apply-templates/></p>
	</xsl:template>

	<xsl:template match="p[@rend='Explanation']">
		<p id="{generate-id()}" class="explanation"><xsl:apply-templates/></p>
	</xsl:template>

	<!--====================================================================-->
	<!-- Line groups -->

	<xsl:template match="lg">
		<p class="poetry"><xsl:apply-templates/></p>
	</xsl:template>

	<xsl:template match="l">
		<br id="{generate-id()}"/>
		<xsl:if test="@n">
			<span class="lineno"><xsl:value-of select="@n"/></span>
		</xsl:if>
		<xsl:apply-templates/>
	</xsl:template>

	<xsl:template match="lb">
		<br id="{generate-id()}"/>
	</xsl:template>

	<!--====================================================================-->
	<!-- Text styles -->

	<!-- TODO: line is actually too high for the intended use. -->
	<xsl:template match="hi[@rend='overline']">
		<span style="text-decoration: overline;"><xsl:apply-templates/></span>
	</xsl:template>

	<!-- TODO: find way to make very wide tildes over words -->
	<xsl:template match="hi[@rend='overtilde']">
		<span style="text-decoration: overline;"><xsl:apply-templates/></span>
	</xsl:template>

	<xsl:template match="hi[@rend='sup']">
		<sup><xsl:apply-templates/></sup>
	</xsl:template>

	<xsl:template match="hi[@rend='sub']">
		<sub><xsl:apply-templates/></sub>
	</xsl:template>

	<xsl:template match="hi[@rend='italic']">
		<i><xsl:apply-templates/></i>
	</xsl:template>

	<xsl:template match="hi[@rend='bold']">
		<b><xsl:apply-templates/></b>
	</xsl:template>

	<xsl:template match="hi[@rend='sc']">
		<span class="smallcaps"><xsl:apply-templates/></span>
	</xsl:template>

	<xsl:template match="hi[@rend='expanded']">
		<span class="letterspaced"><xsl:apply-templates/></span>
	</xsl:template>

	<xsl:template match="hi">
		<i><xsl:apply-templates/></i>
	</xsl:template>

	<!-- Use other font for Greek passages -->
	<xsl:template match="foreign[@lang='el' or @lang='grc']">
		<span class="Greek"><xsl:apply-templates/></span>
	</xsl:template>

	<!-- Use other font for Arabic passages -->
	<xsl:template match="foreign[@lang='ar']">
		<span class="Arabic"><xsl:apply-templates/></span>
	</xsl:template>

	<!--====================================================================-->
	<!-- Measurements with metric equivalent (own extention of TEI) -->

	<xsl:template match="measure">
		<span class="popup">
			<xsl:attribute name="title"><xsl:value-of select="./@reg"/></xsl:attribute>
			<xsl:apply-templates/>
		</span>
	</xsl:template>

	<!--====================================================================-->
	<!-- Project Gutenberg Header, Footer, and License -->

	<xsl:template name="authors">
		<xsl:for-each select="//titleStmt/author">
			<xsl:choose>
				<xsl:when test="position() != last() and last() > 2">
					<xsl:value-of select="."/><xsl:text>, </xsl:text>
				</xsl:when>
				<xsl:when test="position() = last() and last() > 1">
					<xsl:text> </xsl:text><xsl:value-of select="$strAnd"/><xsl:text> </xsl:text><xsl:value-of select="."/>
				</xsl:when>
				<xsl:when test="last() = 1">
					<xsl:value-of select="."/>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="."/>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:for-each>
	</xsl:template>

	<xsl:template name="PGHeader">
		<xsl:variable name="params">
			<params>
				<param name="title"><xsl:value-of select="//titleStmt/title"/></param>
				<param name="authors"><xsl:call-template name="authors"/></param>
				<param name="releasedate"><xsl:value-of select="//publicationStmt/date"/></param>
				<param name="pgnum"><xsl:value-of select="//publicationStmt/idno[@type='pgnum' or @type='PGnum']"/></param>
				<param name="language">
					<xsl:call-template name="GetMessage">
						<xsl:with-param name="name" select="/TEI.2/@lang"/>
					</xsl:call-template>
				</param>
			</params>
		</xsl:variable>
		<xsl:call-template name="FormatMessage">
			<xsl:with-param name="name" select="'msgPGHeader'"/>
			<xsl:with-param name="params" select="$params"/>
		</xsl:call-template>
		<hr/>
		<p/>
	</xsl:template>

	<xsl:template name="PGFooter">
		<xsl:variable name="params">
			<params>
				<param name="title"><xsl:value-of select="//titleStmt/title"/></param>
				<param name="authors"><xsl:call-template name="authors"/></param>
				<param name="transcriber"><xsl:value-of select="//titleStmt/respStmt/name"/></param>
				<param name="pgnum"><xsl:value-of select="//publicationStmt/idno[@type='pgnum' or @type='PGnum']"/></param>
				<param name="pgpath"><xsl:value-of select="substring(//publicationStmt/idno, 1, 1)"/>/<xsl:value-of select="substring(//publicationStmt/idno, 2, 1)"/>/<xsl:value-of select="substring(//publicationStmt/idno, 3, 1)"/>/<xsl:value-of select="substring(//publicationStmt/idno, 4, 1)"/>/<xsl:value-of select="//publicationStmt/idno"/>/</param>
			</params>
		</xsl:variable>
		<p/>
		<hr/>
		<xsl:call-template name="FormatMessage">
			<xsl:with-param name="name" select="'msgPGFooter'"/>
			<xsl:with-param name="params" select="$params"/>
		</xsl:call-template>
		<xsl:call-template name="PGLicense"/>
	</xsl:template>

	<xsl:template name="PGLicense">
		<xsl:call-template name="FormatMessage">
			<xsl:with-param name="name" select="'msgPGLicense'"/>
		</xsl:call-template>
	</xsl:template>

</xsl:stylesheet>
